module datapath (
	input logic clk,
	input logic rst, 
	
	input logic start,
	input logic stop,
	output logic busy,
	output logic success,
	
	input logic mem_done,
	output logic mem_request,
	
	output logic [7:0] e_address,
	input logic [7:0] e_data_out,
	
	input logic [23:0] keyGiven,
	output logic [7:0] testOut
);

s_memory (.clock(clk), .address(address), .data(data_in), .wren(write), .q(data_out));
decrypted_memory (.clock(clk), .address(d_address), .data(d_data_in), .wren(d_write), .q(d_data_out));

logic [7:0] address;
logic [7:0] data_in;
logic [7:0] data_out;
logic write;

logic [7:0] d_address;
logic [7:0] d_data_in;
logic [7:0] d_data_out;
logic d_write;
	
logic [29:0] state;
logic [7:0] i, j, k;
logic [7:0] si, sj, sf, ek;
logic [1:0] imodkey;
logic [23:0] key;

logic k_flag;
assign testOut = {{2'b0},ek};
//assign key = keyGiven;

logic [7:0] key2;
logic [7:0] key1;
logic [7:0] key0;

assign key2 = key[7:0];
assign key1 = key[15:8];
assign key0 = key[23:16];

assign write = state[0];
assign d_write = state [24];
assign busy = ~state[27];
assign success = state[28];

assign mem_request = state[29];

assign imodkey = i%3;
//BIT EXPLANATION
//state[0] = write;

//state[1] = address is i
//state[2] = address is j

//state[3] = reset j/i/k

//state[4] = increment i

//state[5] = j is j+si+key0
//state[6] = j is j+si+key1
//state[7] = j is j+si+key2

//state[8] = data_in is i
//state[9] = data_in is j
//state[10] = data_in is si
//state[11] = data_in is sj

//state[12] = si is data_out
//state[13] = sj is data_out

//state[14] = resets k, i and j

//state[15] = turns on k flag when high
//state[16] = turns off k flag when high

//state[17] = set j as j+si

//state[18] = address updates to si plus sj
//state[19] = e_addr updates with k
//state[20] = d_addr updates with k

//state[21] = sf is data_out
//state[22] = ek is e_data_out

//state[23] = d_data_in is sf XOR ek
//state[24] = write to d

//state[25] = increments k

//state[26] = update key

//state[27] = busy
//state[28] = success

//state[29] = mem_request
parameter RESET = 30'b0_01_0_0_00_00_000_0_10_0_00_0000_000_0_1_00_0;
parameter UPDATEKEY = 30'b0_00_1_0_00_00_000_0_00_0_00_0000_000_0_0_00_0;
parameter INIT = 30'b00_0001_000_1_0_01_1;
parameter RSTVAR = 30'b00_0000_000_0_1_00_1;

parameter SETI = 30'b00_0000_000_1_0_00_0;
parameter READI = 30'b00_0000_000_0_0_01_0;
parameter WAITI = 30'b11_0000_000_0_0_00_0;
parameter PULLSI = 30'b01_0000_000_0_0_00_0;

parameter SETJKEY0 = 30'b00_0000_001_0_0_00_0;
parameter SETJKEY1 = 30'b00_0000_010_0_0_00_0;
parameter SETJKEY2 = 30'b00_0000_100_0_0_00_0;
parameter READJ = 30'b00_0100_000_0_0_10_0;
parameter WAITJ = 30'b10_0100_000_0_0_00_0;
parameter PULLSJ = 30'b10_0000_000_0_0_00_0;
parameter SETSJ = 30'b00_1000_000_0_0_01_1;
parameter SETSI = 30'b00_0000_000_0_0_00_1;
parameter IDLE = 30'b0_11_0_0_00_00_000_0_00_0_00_0000_000_0_0_00_0;

parameter RESETIJK = 30'b1_1_00_0000_000_0_1_00_0;
parameter ADDI = 30'b1_0_00_0000_000_1_0_00_0;
parameter SETJ = 30'b1_00_0_00_0000_000_0_0_00_0;
parameter READKF =             30'b0_00_101_0_00_0_00_0000_000_0_0_00_0;
parameter PULSEMEM = 30'b1_00_0_0_00_00_111_0_00_0_00_0000_000_0_0_00_0;
parameter PULLES =             30'b0_11_000_0_00_0_00_0000_000_0_0_00_0;
parameter LOADDK =             30'b1_00_000_0_00_0_00_0000_000_0_0_00_0;
parameter SETDK = 30'b10_00_000_0_00_0_00_0000_000_0_0_00_0;
parameter WAITDK = 30'b0;
parameter ADDK = 30'b1_00_00_000_0_00_0_00_0000_000_0_0_00_0;
always_ff @ (posedge clk or negedge rst) begin
	if (!rst)
		state <= RESET;
	else if (stop)
		state <= IDLE;
	else begin
		case (state)
			RESET: 
				if(start)
					state <= UPDATEKEY;
				else
					state <= RESET;
			UPDATEKEY: state <= INIT;
			INIT:
				if(i == 8'd255)
					state <= RSTVAR;
				else
					state <= INIT;
			RSTVAR: state <= READI;
			READI: state <= WAITI;
			WAITI: state <= PULLSI;
			PULLSI:
				if(k_flag)
					state <= SETJ;
				else if(imodkey == 0)
					state <= SETJKEY0;
				else if (imodkey == 1)
					state <= SETJKEY1;
				else
					state <= SETJKEY2;
			SETJKEY0: state <= READJ;
			SETJKEY1: state <= READJ;
			SETJKEY2: state <= READJ;
			READJ: state <= WAITJ;
			WAITJ: state <= PULLSJ;
			PULLSJ: state <= SETSJ;
			SETSJ: state <= SETSI;
			SETSI: 
				if(k_flag)
					state <= PULSEMEM;
				else if(i == 8'd255)
					state <= RESETIJK;
				else
					state <= SETI;
			SETI: state <= READI;
			IDLE: state <= IDLE;
			
			RESETIJK: state <= ADDI;
			ADDI: state <= READI;
			SETJ: state <= READJ;

			PULSEMEM: state <= READKF;
			READKF: 
				if (mem_done)
					state <= PULLES;
				else
					state <= READKF;
			
			PULLES: state <= LOADDK;
			LOADDK: state <= WAITDK;
			WAITDK: 
				if((d_data_in == 8'd32)|((d_data_in >= 8'd97) && (d_data_in <= 8'd122)))
					state <= SETDK;
				else
					state <= RESET;
			SETDK:
				if(k == 8'd31)
					state <= IDLE;
				else
					state <= ADDK;
			ADDK: state <= ADDI;
			default: state <= INIT;
	
		endcase 
	
	
	
	end

end

//BIT EXPLANATION
//state[0] = write;

//state[1] = address is i
//state[2] = address is j

//state[3] = reset j/i/k

//state[4] = increment i

//state[5] = j is j+si+key0
//state[6] = j is j+si+key1
//state[7] = j is j+si+key2

//state[8] = data_in is i
//state[9] = data_in is j
//state[10] = data_in is si
//state[11] = data_in is sj

//state[12] = si is data_out
//state[13] = sj is data_out

//state[14] = resets k, i and j

//state[15] = turns on k flag when high
//state[16] = turns off k flag when high

//state[17] = set j as j+si

//state[18] = address updates to si plus sj
//state[19] = e_addr updates with k
//state[20] = d_addr updates with k

//state[21] = sf is data_out
//state[22] = ek is e_data_out

//state[23] = d_data_in is sf XOR ek
//state[24] = write to d

//state[25] = increments k

always_ff @ (posedge clk) begin
	if (state[26])
		key <= keyGiven;
end

always_ff @ (posedge clk) begin
	if (state[23])
		d_data_in <= sf^ek;
end

always_ff @ (posedge clk) begin
	if (state[22])
		ek <= e_data_out;
end

always_ff @ (posedge clk) begin
	if (state[19])
		e_address <= k;
end

always_ff @ (posedge clk) begin
	if (state[20])
		d_address <= k;
end

always_ff @ (posedge clk) begin
	if (state[21])
		sf <= data_out;
end
always_ff @ (posedge clk) begin
	if(state[1])
		address <= i;
	else if (state[2])
		address <= j;
	else if (state[18])
		address <= si+sj;
end

always_ff@(posedge clk) begin
	if (state[3]) 
		i <= 8'b0;
	else if (state[4])
		i <=i+1;
end

always_ff @ (posedge clk) begin
	if (state[3]) 
		j <= 8'b0;
	else if (state[5])
		j <= (j + si +key0);
	else if (state[6])
		j <= (j + si +key1);
	else if (state[7])
		j <= (j + si +key2);
	else if (state[17])
		j <= j+si;
end 


always_ff @ (posedge clk) begin
	if (state[8])
		data_in <= i;
	else if (state[9])
		data_in <= j;
	else if (state[10])
		data_in <= si;
	else if (state[11]) 
		data_in <= sj;
end 

always_ff @ (posedge clk) begin
	if (state[12])
		si <= data_out;
	else if (state[13])
		sj <= data_out;
end

always_ff @ (posedge clk) begin
	if (state[14])
		k <= 8'b0;
	else if (state[25])
		k <= k+1;
end

always_ff @ (posedge clk) begin
	if (state[15])
		k_flag <= 1'b1;
	else if (state[16])
		k_flag <= 1'b0;
end
endmodule
