module encrypted_access_controller (
	input logic a_req,
	input logic b_req,
	input logic c_req,
	input logic d_req,
	
	input logic reset,
	input logic clk,
	
	output logic clrA,
	output logic clrB,
	output logic clrC,
	output logic clrD,
	
	output logic a_done,
	output logic b_done,
	output logic c_done,
	output logic d_done,
	
	input logic [7:0] a_address,
	input logic [7:0] b_address,
	input logic [7:0] c_address,
	input logic [7:0] d_address,

	output logic [7:0] data_out,

	input logic [7:0] e_data,
	output logic [7:0] e_address
);
logic [11:0] state;

assign a_done = state[2];
assign b_done = state[3];
assign c_done = state[4];
assign d_done = state[5];

assign clrA = state[8];
assign clrB = state[9];
assign clrC = state[10];
assign clrD = state[11];
//0_00_0000_000
//state[9] = Wait?
//state[7] = update address
//state[6] = update datastream
parameter CHECKA = 12'b0_10_0000_00;
parameter WAITA =	12'b1_00_0000_00;
parameter PUSHA=	12'b0_01_0000_00;
parameter FINISHA=12'b0_00_0001_00;

parameter CHECKB = 12'b0_10_0000_01;
parameter WAITB =	12'b10_00_0000_01;
parameter PUSHB=	12'b0_01_0000_01;
parameter FINISHB=12'b0_00_0001_01;

parameter CHECKC = 12'b0_10_0000_10;
parameter WAITC =	12'b100_00_0000_10;
parameter PUSHC=	12'b0_01_0000_10;
parameter FINISHC=12'b0_00_0001_10;

parameter CHECKD = 12'b0_10_0000_11;
parameter WAITD =	12'b1000_00_0000_11;
parameter PUSHD=	12'b0_01_0000_11;
parameter FINISHD=12'b0_00_0001_11;

always_ff @ (posedge clk or negedge reset) begin
	if (!reset)
		state <= CHECKA;
	else begin
		case(state)
			CHECKA: 
				if(a_req)
					state <= WAITA;
				else
					state <= CHECKB;
			WAITA: state <= PUSHA;
			PUSHA: state <= FINISHA;
			FINISHA: state <= CHECKB;
			
			CHECKB: 
				if(b_req)
					state <= WAITB;
				else
					state <= CHECKC;
			WAITB: state <= PUSHB;
			PUSHB: state <= FINISHB;
			FINISHB: state <= CHECKC;
			
			CHECKC: 
				if(c_req)
					state <= WAITC;
				else
					state <= CHECKD;
			WAITC: state <= PUSHC;
			PUSHC: state <= FINISHC;
			FINISHC: state <= CHECKD;
			
			CHECKD: 
				if(d_req)
					state <= WAITD;
				else
					state <= CHECKA;
			WAITD: state <= PUSHD;
			PUSHD: state <= FINISHD;
			FINISHD: state <= CHECKA;
			default: state <= CHECKA;
				
		endcase
	end
end

always_ff@(posedge clk) begin
	if((state[7]) && (state[1:0] == 2'b0))
		e_address <= a_address;
	else if((state[7]) && (state[1:0] == 2'b01))
		e_address <= b_address;
	else if((state[7]) && (state[1:0] == 2'b10))
		e_address <= c_address;
	else if((state[7]) && (state[1:0] == 2'b11))
		e_address <= d_address;
end

always_ff@(posedge clk) begin
	if((state[6]))
		data_out <= e_data;

		
end

endmodule